#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Collections.Generic;
using System.Net;
using System.IO;
using System.Threading;
#endregion


/*

	The TradingPhysicsConnection namespace holds connectivity wrapper to www.tradingphysics.com

	For more information on how to use this plugin:
	http://www.tradingphysics.com/Resources/Integration/NinjaTrader/

	For more information about the open API, please go to:
	http://www.tradingphysics.com/Resources/Specifications/

	The lists of indicators and oscillators are located here:
	http://www.tradingphysics.com/Resources/Specifications/MarketIndicatorsList.aspx
	http://www.tradingphysics.com/Resources/Specifications/OscillatorsList.aspx

	Download the latest code from here:
	http://www.tradingphysics.com/Resources/Tools/

	For all technical questions/issues, please email at support@tradingphysics.com

	*** IMPORTANT NOTE ***

	Please enter your customer number and password hash code into the code in TradingPhysicsIndicator.Indicator:
	
	line 768: static String MyCustomerNumber = ""; // customer number in the NNN-NNN-NNN format
    line 769: static String MyHashCode = ""; // password hash code 

	In order to receive free credits, go to https://www.tradingphysics.com/User/Register.aspx
	This information is available on your Account Information Page
	Leave empty strings to use evaluation credits with limited access (one ticker)
        
	(C) 2008-2011 www.tradingphysics.com

	This Version 2.3 for NT BETA 7.20 as of September 8, 2010

*/

namespace TradingPhysicsConnection
{

    class Session
    {
        // delegate type for hooking up incoming data updates for raw CSV rows, including header
        public delegate void OnRowUpdateEventHandler(Session Sender, string ARow);
        public event OnRowUpdateEventHandler OnRowUpdateEvent;

        // delegate type for hooking up incoming data updates for actual data
        public delegate void OnTickUpdateEventHandler(Session Sender, int ATickIndex);
        public event OnTickUpdateEventHandler OnTickUpdateEvent;

        // valid connection states
        public enum ConnectionStates { csDisconnected, csConnected, csAborted, csFinished };

        // session start in seconds after midnight, 7:00 AM EST
        private int SessionStart = 25200;

        // address of the API server
        public static string ServerUrl = "http://api.tradingphysics.com/";

        // ticket query string template
        private static string GetTicketQueryTemplate = "getticket?C={0}&P={1}?{2}";
        
		// actual query
        private string TicketQuery;

        // data query string template
        private static string GetDataQueryTemplate = "getdata?type={0}&format=csv&compression=none&keepconnection=true&tick={1}&date={2}&stock={3}&{0}={4}";
		
        // actual query
        private string DataQuery;

        // authentication info
        private string CustomerNumber;
        private string HashCode;
		private string DataType; // oscillators or indicators

        // session info 
        private ConnectionStates ConnectionState;
        private string Ticket;
        private string Date; // date in YYYYMMDD format
        private string Stock; // stock symbol
        private int TickTime; // tick size in milliseconds
        private int TickCount; // how many ticks have been retrieved
        private List<string> Indicators; // array of indicators
        private string IndicatorsQuery; // part of the url containing the list of indicators joined by "+"

        // dynamically allocated array of doubles associated with the name of the indicator
        Dictionary<string, List<double>> Data;

        // update thread
        Thread UpdateThread;
        ManualResetEvent Realtime;

        // error message while reading 
        string ErrorMessage;

        //
        // Private Methods
        //

        // prepares data members before downloading data
        private void InitData()
        {
            // check input data 
            if (Indicators.Count == 0) throw new Exception("Please add indicators you wish to retrieve");

            // estimate total data size based on tick size
            int RowCount;
            switch (TickTime)
            {
                case 1000:
                    RowCount = 46800;
                    break;

                case 5000:
                    RowCount = 9360;
                    break;

                case 60000:
                    RowCount = 780;
                    break;

                default:
                    throw new Exception("Invalid tick time (expecting 1000, 5000, or 60000)");
            }

            // allocate array of doubles for each indicator
            Data = new Dictionary<string, List<double>>(Indicators.Count);

            // add arrays of indicator values
            foreach (string n in Indicators)
            {
                Data.Add(n, new List<double>(RowCount));
            }

            // initialize starting tick with -1 (no data received)
            TickCount = -1;
        }

        // sets connection status and error message, if any
        private void ChangeConnectionState(string AErrorMessage, ConnectionStates AConnectionState)
        {
            lock (ErrorMessage)
            {
                ErrorMessage = AErrorMessage;
                ConnectionState = AConnectionState;
            }
        }

        // reads and parses data from the web site
        private void ReadData()
        {
            try
            {
                // request the data itself
                // if transaction is payable, the credits will be charged only upon first ticket usage
                // the url with the same ticket may be reused without additional charges
                HttpWebRequest r = (HttpWebRequest)WebRequest.Create(ServerUrl + DataQuery);
                try
                {
					r.Timeout = 60000;
					r.ReadWriteTimeout = 60000;
                    HttpWebResponse rr = (HttpWebResponse)r.GetResponse();
                    if (rr.StatusCode != HttpStatusCode.OK) 
					{
						throw new Exception("HTTP Error: " + rr.StatusDescription);
					}
					try
					{
						// get data from response stream if transaction was successful
						// we assume that the data was retrieved in text format so that we can print it on the console window
						Stream ss = rr.GetResponseStream();
						StreamReader Reader = new StreamReader(ss);
						while (!Reader.EndOfStream)
						{
							ParseLine(Reader.ReadLine());
						}
	
						// set successful finish status
						ChangeConnectionState("", ConnectionStates.csFinished);
					} 
					finally
					{
						rr.Close();
					}
                }
                catch (WebException e)
                {
                    string s = e.Message;

					// read error description from response stream
                    if (e.Response != null)
                    {
                        StreamReader Reader = new StreamReader(e.Response.GetResponseStream());
                        s += " (Server Error: " + Reader.ReadToEnd() + ")";
                    }
                    ChangeConnectionState(s, ConnectionStates.csAborted);
                }
                catch (Exception e)
                {
                    ChangeConnectionState(e.Message, ConnectionStates.csAborted);
                }
            }
            finally
            {
                // set signal to release the waiting thread
                Realtime.Set();
            }
        }

        // parses one line (table row)
        private void ParseLine(string ARow)
        {
            // ignore heartbeats / end-of-file messages
            // presence of heartbeats indicates real-time feed
            // the server always sends a heartbeat 
            // as soon as the client catches up with the real time feed
            if (ARow == "")
            {
                Realtime.Set();
                return;
            }

            // split record fields using "," delimiter
            string[] Columns = ARow.Split(new char[] { ',' });

            // determine the number of received columns, including tick timestamp and trailing ","
            int n = Columns.Length;
            if (n != Indicators.Count + 2) throw new Exception("The number of columns does not match the number of indicators");

            lock (Data)
            {
                if (TickCount < 0)
                {
                    // parse header
                    for (int i = 0; i < n - 1; i++)
                    {
                        string s = Columns[i];
                        if (i == 0)
                        {
                            if (s != "Tick") throw new Exception("Invalid header received: expecting \"Tick\" at column 0");
                        }
                        else
                        {
                            if (Indicators[i - 1] != s)
                            {
                                string e = String.Format(
                                    "Invalid header received: expecting column {0} to contain {1}, not {2}",
                                    i, Indicators[i], s);

                                throw new Exception(e);
                            }
                        }
                    }
                }
                else
                {
                    // parse body
                    for (int i = 0; i < n - 1; i++)
                    {
                        string s = Columns[i];
                        if (i == 0)
                        {
                            // convert expected tick timestamp in HH:MM:SS format 
                            string z = GetTickTimestamp(TickCount);
                            if (z != s) throw new Exception(String.Format("Tick timestamp out of sync: expecting {0}, received {1}", z, s));
                        }
                        else
                        {
                            double d = double.Parse(s);
                            Data[Indicators[i - 1]].Add(d);
                        }
                    }
                }

                TickCount++;
            }
			
			// propagate events
            if (OnRowUpdateEvent != null) OnRowUpdateEvent(this, ARow);
            if (TickCount > 0)
			{
                if (OnTickUpdateEvent != null)
				{
                    OnTickUpdateEvent(this, TickCount - 1);
				}
			}
        }

        //
        // Public Methods
        //

        // class constructor
        public Session(string ACustomerNumber, string AHashCode, string ADataType, string AStock, string ADate, int ATickTime)
        {
            // connection parameters
            CustomerNumber = ACustomerNumber;
            HashCode = AHashCode;
			DataType = ADataType;
            Stock = AStock;
            Date = ADate;
            TickTime = ATickTime;

            // list of indicators
            Indicators = new List<string>();
            IndicatorsQuery = "";

            // connection state and thread error message
            ConnectionState = ConnectionStates.csDisconnected;
            ErrorMessage = "";

            // will be set as soon as catch up with realtime
            Realtime = new ManualResetEvent(false);
        }

        // class destructor
        ~Session()
        {
            Abort();
        }

        // adds indicator name -- must be called prior to Connect()
        public void AddIndicator(string AIndicator)
        {
            // check for existing connection
            if (GetConnectionState() != ConnectionStates.csDisconnected)
                throw new Exception("Cannot add indicators after the connection has been established");

            // add indicator to the list
            Indicators.Add(AIndicator);

            // add indicator to the query string
            IndicatorsQuery += (IndicatorsQuery == "") ? AIndicator : "+" + AIndicator;
        }

        // connects to the www.tradingphysics.com data server and launches the data retrieval
        public void Connect()
        {
            // check connection
            if (GetConnectionState() == ConnectionStates.csConnected) throw new Exception("Already connected");
            if (GetConnectionState() != ConnectionStates.csDisconnected) throw new Exception("Reconnect feature is not implemented");

            // initialize data
            InitData();

            // initialize queries to the trading physics web site
            DataQuery = String.Format(GetDataQueryTemplate, DataType, TickTime, Date, Stock, IndicatorsQuery);
            TicketQuery = String.Format(GetTicketQueryTemplate, CustomerNumber, HashCode, DataQuery);

            try
            {
                // request ticket first
                HttpWebRequest r = (HttpWebRequest)WebRequest.Create(ServerUrl + TicketQuery);
				r.Timeout = 5000;
				r.ReadWriteTimeout = 5000;
                HttpWebResponse rr = (HttpWebResponse)r.GetResponse();
                if (rr.StatusCode != HttpStatusCode.OK) 
				{
					throw new Exception("HTTP Error: " + rr.StatusDescription);
				}
				try
				{
	                // get ticket from response stream if transaction was successful
    	            StreamReader Reader = new StreamReader(rr.GetResponseStream());
                	Ticket = Reader.ReadToEnd();
				}
				finally
				{
					rr.Close();
				}
            }
            catch (WebException e)
            {
                string s = e.Message;

                // read error description from response stream
                if (e.Response != null)
                {
                    StreamReader Reader = new StreamReader(e.Response.GetResponseStream());
                    s += " (Server Error: " + Reader.ReadToEnd() + ")";
                }

				// remember error message
				ChangeConnectionState(s, ConnectionStates.csAborted);
				
				// rethrow as standard exception
                throw new Exception(s);
            }
            catch (Exception e)
            {
				// remember error message
				ChangeConnectionState(e.Message, ConnectionStates.csAborted);
				
				// rethrow 
                throw new Exception(ErrorMessage);
            }

            // update query to include the ticket (ticket may be an empty string if no ticket required for this request)
            if (Ticket != "") DataQuery += "&t=" + Ticket;

            // change connection status, read data in an update thread
            ChangeConnectionState("", ConnectionStates.csConnected);
            UpdateThread = new Thread(ReadData);
            UpdateThread.Start();
        }

        // waits till all reading is complete by the update thread, and the thread terminates
        // this will not return until end of session if requesting real-time data
        // for historical data, will return as soon as all data has been loaded
        public void Join()
        {
            if (UpdateThread != null)
                if (GetConnectionState() == ConnectionStates.csConnected)
                    UpdateThread.Join();
        }

        // abort connection
        public void Abort()
        {
            if (UpdateThread != null)
            {
                if (GetConnectionState() == ConnectionStates.csConnected)
                {
                    // there is a tiny chance that the thread terminates in between
                    // in this case, avoid possible exception if aborting a terminated thread
                    try
                    {
                        UpdateThread.Abort();
                    }
                    catch
                    {
                        // ignore all errors
                    }

                    // wait till the thread terminates
                    UpdateThread.Join();
                }
            }
        }

        public bool IsConnected()
        {
            return (GetConnectionState() == ConnectionStates.csConnected);
        }

        public void CatchUp()
        {
            Realtime.WaitOne();
        }

        //
        // Public Properties
        //

        // returns the number of retrieved ticks
        // 0: no data has been received
        // -1: no header has been received
        public int GetTickCount()
        {
            lock (Data)
            {
                return TickCount;
            }
        }

        // returns the indicator count
        public int GetIndicatorCount() { return Indicators.Count; }

        // returns the name of indicator by its index
        // indicator index = 0..Indicators.Count-1
        public string GetIndicatorName(int AIndicatorIndex) { return Indicators[AIndicatorIndex]; }

        // return indicator value by tick index, 
        // tick index = 0..TickCount-1
        public double GetIndicatorValue(string AIndicatorName, int ATickIndex)
        {
            // obtain indicator index
            int i = Indicators.IndexOf(AIndicatorName);
            if (i < 0) throw new Exception("Unknown indicator: \"" + AIndicatorName + "\"");

            lock (Data)
            {
                // check ranges for tick count
                if ((ATickIndex >= TickCount) || (ATickIndex < 0))
                {
                    string e = String.Format("Tick index out of range in GetIndicatorValue(\"{0}\", {1}): TickCount = {3}",
                        Indicators[i], ATickIndex, TickCount);
                    throw new Exception(e);
                }
                return Data[Indicators[i]][ATickIndex];
            }
        }

        // thread-safely return the value of the indicator for the given indicator and tick indexes
        // IndicatorIndex = 0..Indicators.Count-1
        // TickIndex = 0..TickCount-1 
        public double GetIndicatorValue(int AIndicatorIndex, int ATickIndex)
        {
            // check ranges for indicator index
            if ((AIndicatorIndex >= Indicators.Count) || (AIndicatorIndex < 0))
            {
                string e = String.Format("Indicator out of range in GetIndicatorValue(\"{0}\", {1}): IndicatorCount = {3}",
                    AIndicatorIndex, ATickIndex, Indicators.Count);
                throw new Exception(e);
            }

            lock (Data)
            {
                // check ranges for tick count
                if ((ATickIndex >= TickCount) || (ATickIndex < 0))
                {
                    string e = String.Format("Tick index out of range in GetIndicatorValue(\"{0}\", {1}): TickCount = {3}",
                        Indicators[AIndicatorIndex], ATickIndex, TickCount);
                    throw new Exception(e);
                }
                return Data[Indicators[AIndicatorIndex]][ATickIndex];
            }
        }

        // returns a clean timestamp for the tick in the HH:MM:SS string format
        public string GetTickTimestamp(int ATickIndex)
        {
            // convert tick index to timestamp, in seconds after midnight
            int t = (ATickIndex + 1) * (TickTime / 1000) + SessionStart;

            // convert to HH:MM:SS format
            int s = t % 60;
            int m = (t / 60) % 60;
            int h = (t / 60 / 60) % 60;
			
            return h.ToString("D2") + ":" + m.ToString("D2") + ":" + s.ToString("D2");
        }

        // converts DateTime type to tick index = 0..TickCount-1 (date info is ignored)
        public int ConvertDateTimeToTickIndex(DateTime t)
        {
            // obtain seconds after midnight for received tick
            int t1 = t.Second + t.Minute * 60 + t.Hour * 60 * 60;

            // convert seconds to index
            int t2 = (t1 - SessionStart) / (TickTime / 1000) - 1;

            // return -1 if out of range
            if (t2 < 0) return -1;
            if (t2 >= TickCount) return -1;
            return t2;
        }

        // converts tick index = 0..TickCount-1 to DateTime type (date info is ignored)
        public DateTime ConvertTickIndexToDateTime(int ATickIndex)
        {
            // convert tick index to timestamp, in seconds after midnight
            int t = (ATickIndex + 1) * (TickTime / 1000) + SessionStart;

            // convert to HH:MM:SS format
            int s = t % 60;
            int m = (t / 60) % 60;
            int h = (t / 60 / 60) % 60;
			
            return new DateTime(0, 0, 0, s, m, h);
        }
		
        // thread-safe error message retrieval (empty string "" in case of no errors)
        public string GetErrorMessage()
        {
            lock (ErrorMessage)
            {
                return ErrorMessage;
            }
        }

		// thread-safe error message reset
        public void ClearErrorMessage()
        {
            lock (ErrorMessage)
            {
                ErrorMessage = "";
            }
        }

		// thread-safe connection state retrieval
        public ConnectionStates GetConnectionState()
        {
            lock (ErrorMessage)
            {
                return ConnectionState;
            }
        }

        // session date
        public string GetSessionDate()
        {
            return Date;
        }
    }

    class SessionManager
    {
        // events
        public event Session.OnRowUpdateEventHandler OnRowUpdateEvent;
        public event Session.OnTickUpdateEventHandler OnTickUpdateEvent;
        
        // authentication info
        private string CustomerNumber;
        private string HashCode;
		private string DataType; // oscillators or indicators

        // session info 
        private string Stock; // stock symbol
        private int TickTime; // tick size in milliseconds
        private List<string> Indicators; // array of indicators

        // dynamically allocated array of sessions by session date
        Dictionary<string, Session> Sessions;

        // class constructor
        public SessionManager(string ACustomerNumber, string AHashCode, string ADataType, string AStock, int ATickTime, int AMaxConcurrentConnections)
        {
			// by default, the .Net allows only 2 concurrent connections to the same site:port
			// you might want to request several indicators for the same session at once to reduce overhead
			ServicePoint sp = ServicePointManager.FindServicePoint(new Uri(Session.ServerUrl));
			if (sp != null) sp.ConnectionLimit = AMaxConcurrentConnections;

			// connection parameters
            CustomerNumber = ACustomerNumber;
            HashCode = AHashCode;
			DataType = ADataType;
            Stock = AStock;
            TickTime = ATickTime;

            // list of indicators
            Indicators = new List<string>();

            // list of new sessions by session date
            Sessions = new Dictionary<string, Session>();
        }

        // adds indicator name
        public void AddIndicator(string AIndicator)
        {
            // check for existing sessions
            if (Sessions.Count > 0) throw new Exception("Cannot add indicators if there already exists at least one session");

            // add indicator to the list
            Indicators.Add(AIndicator);
        }

        // checks if this session exists
        public bool HasSession(string ADate)
        {
            return Sessions.ContainsKey(ADate);
        }

        // checks if this session exists
        public bool HasSession(DateTime ADate)
        {
            string d = ADate.ToString("yyyyMMdd");
            return Sessions.ContainsKey(d);
        }

        // add a new session
        public Session AddSession(string ADate)
        {
            // check if any indicators are requested
            if (Indicators.Count == 0) throw new Exception("Cannot add session if there are no indicators to retrieve");
            
            // return existing session
            if (Sessions.ContainsKey(ADate)) return Sessions[ADate];

            // add a new session
            Session s = new Session(CustomerNumber, HashCode, DataType, Stock, ADate, TickTime);

            // add indicators to the session
            foreach (string IndicatorName in Indicators)
            {
                s.AddIndicator(IndicatorName);
            }

            // add handlers to the session
            if (OnRowUpdateEvent != null) s.OnRowUpdateEvent += OnRowUpdateEvent;
            if (OnTickUpdateEvent != null) s.OnTickUpdateEvent += OnTickUpdateEvent;

            // add session to the list 
            // call Connect() later
            Sessions.Add(ADate, s);
 
			// return the new session
            return s;
        }

        // add a new session
        public Session AddSession(DateTime ADate)
        {
            string d = ADate.ToString("yyyyMMdd");
            return AddSession(d);
        }

        // access a new session if it exists
        public Session GetSession(string ADate)
        {
            // throw an exception if session not found
            if (!Sessions.ContainsKey(ADate)) throw new Exception("Session \"" + ADate + "\" not found");

            // return existing session 
            return Sessions[ADate];
        }

        // access a new session if it exists
        public Session GetSession(DateTime ADate)
        {
            string d = ADate.ToString("yyyyMMdd");
            return GetSession(d);
        }

        // returns the number of active sessions
        public int GetSessionCount()
        {
            return Sessions.Count;
        }
		
        // abort realtime connections
        public void Abort()
        {
            foreach (Session s in Sessions.Values)
            {
                s.Abort();
            }
        }
    }
}

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Order-flow based indicators from www.tradingphysics.com
    /// </summary>
    [Description("Order-flow based indicators from www.tradingphysics.com")]
    public class TradingPhysicsIndicator : Indicator
    {
        #region Variables
        // Wizard generated variables
            private string aName = @""; // Default setting for AName
        // User defined variables (add any user defined variables below)
        #endregion

        // authentication info -- initialize it with valid account information
		// to receive free credits, go to https://www.tradingphysics.com/User/Register.aspx
		// this information is available on your Account Information Page
		// leave empty strings to use evaluation credits with limited access to 
        static String MyCustomerNumber = ""; // customer number in the NNN-NNN-NNN format
        static String MyHashCode = ""; // password hash code 
		
		// the data series of the indicator
		private DataSeries MyDataSeries;
		
		// this string will contain an error that occured during interactions with www.traingphysics.com web site or "" if everything is fine
		private String ErrorMessage;
		
		// session manager
		private TradingPhysicsConnection.SessionManager MySessionManager;

		// the name of the instrument for which to request indicators
		private String MyInstrumentName;

		// the indicator name (requesting only one, but may request up to 400+ indicators or 200+ oscillators at the same time)
		private String MyIndicatorName;

		// what are we requesting: "indicators" or "oscillators"?
		private String MyDataType;
		
		// counts how many bars have been loaded (the oldest is 0, the latest is MyTimeSeries.Count-1)
		private int LoadedBarCount;

		// status of a bar update attempt
		private enum BarUpdateResult { SUCCESS, STILL_LOADING, NOT_AVAILABLE }
		
		// use this method to obtain the error message (e.g. connection error)
		public string GetErrorMessage() { return ErrorMessage; }
		
		/// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Navy), PlotStyle.Line, "Plot0"));
			
			// update on every incoming tick to ensure all observations are initialized!
            CalculateOnBarClose	= false; 
			
			// other chart paramters
            Overlay				= false;
			PriceTypeSupported	= false;

			// this string will contain an error that occured during interactions with www.traingphysics.com web site or "" if everything is fine
			ErrorMessage = "";
			
			// counts how many bars have been loaded from www.tradingphysics.com
			// (the oldest is 0, the latest is MyTimeSeries.Count-1)
			LoadedBarCount = 0;

			// instrument name will be initialized later in OnBarUpdate
			MyInstrumentName = "";	
			
			// remember and print the error message in case of an exception
			try
			{
				// this object will hold the values for the indicator
				MyDataSeries = new DataSeries(this);
	
				// the indicator name (requesting only one, but may request up to 400+ at the same time)
				MyIndicatorName = aName.Trim();
				if (MyIndicatorName == "") throw new Exception("Name parameter must be set to a valid oscillator or indicator name");
				
				// either indicator or oscillator types are allowed at this time
				MyDataType = MyIndicatorName.Trim().ToLower();
				if (MyDataType.IndexOf("oscillator.") == 0) 
					MyDataType = "oscillators"; 
				else if (MyDataType.IndexOf("marketindicator.") == 0) 
					MyDataType = "indicators"; 
				else
					throw new Exception("Invalid name: \"" + MyIndicatorName + "\". Please refer to www.tradingphysics.com for the lists of valid indicator and oscillator names.");
			}
			catch (Exception e)
			{
				// report exception to Output Console
				Print("Error in Initialize(): " + e.Message);
				
				// remember the error
				ErrorMessage = e.Message;
				
				// throw the error again
				throw;
			}
        }
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			// don't update data if an exception occured
			if (ErrorMessage != "") 
			{
				Print(ErrorMessage);
				return;
			}
			
			// don't initialize until there are at least two bars 
			// (to determine optimal tick time for request)
			if (MyDataSeries.Count < 2) return;
			
			// update data
			try
			{
				// initialize instrument name (not always available in Initialize)
				if (MyInstrumentName == "")
				{
					// the name of the instrument for which to request indicators
					try
					{
						MyInstrumentName = Instrument.FullName;
					}
					catch (Exception e)
					{
						Print("Exception at MyInstrumentName = Instrument.FullName: " + e.Message);
						throw;
					}

					// determine optimal tick time
					// request per-minute data by default
					TimeSpan t = Time[0] - Time[1];
					int TickTime = 60 * 1000;
					if (t.TotalSeconds < 60)
					{
						if ((t.TotalSeconds % 5) == 0)
							TickTime = 5000;
						else
							TickTime = 1000;
					}
					
					// initialize session manager
					try
					{
						// this stores various dates, including at least one for which the chart is being painted
						MySessionManager = new TradingPhysicsConnection.SessionManager(MyCustomerNumber, MyHashCode, MyDataType, MyInstrumentName, TickTime, 100 /* max concurrent connections */);
					}
					catch (Exception e)
					{
						Print("Exception at TradingPhysicsConnection.SessionManager(): " + e.Message);
						throw;
					}
					
					// add indicator to the manager that way all sessions will know what indicators to retrieve from the server
					try
					{
						MySessionManager.AddIndicator(MyIndicatorName);
					}
					catch (Exception e)
					{
						Print("Exception at MySessionManager.AddIndicator(): " + e.Message);
						throw;
					}
				}
	
				// if true, then we are missing bars because the session is still loading in the background
				// we will have to request them next time the OnBarUpdate is called
				// if false, then the bars are not available at all, so don't request them the second time and just keep them uninitialized
				bool CatchingUp = false;

				// if all bars are loaded, then just update the last one,
				// otherwise load remaining bars (MyDataSeries.Count - LoadedBars)
				if (MyDataSeries.Count == LoadedBarCount)
				{
					// ignore result of the update operation
					TryUpdateBar(0);
				}
				else 
				{
					for (int i = MyDataSeries.Count - 1 - LoadedBarCount; i>= 0; i--)
					{
						// try setting indicator values for missing bars from tradingphysics
						BarUpdateResult r = TryUpdateBar(i); 
						if (r != BarUpdateResult.SUCCESS)
						{
							// why is this tick unavailable?
							// if catching up, don't set this tick's value -- will attempt to load it later
							if (!CatchingUp)
							{
								// because still loading data?
								if (r == BarUpdateResult.STILL_LOADING)
								{
									// yes -- request this bar later on the next OnBarUpdateEvent 
									// LoadedBars will stop increasing for this OnBarUpdateEvent, 
									// the next OnBarUpdateEvent will start updating values from this bar
									CatchingUp = true;
								}
								else
								{
									// the session is fully loaded and no data is expected from www.tradingphysics.com
									// the tick is probably outside of the trading hours, 
									// so remember a missed value for it by not setting any value at all
									LoadedBarCount++;
								}
							}
						}
					}
				}
			}
			catch (Exception e)
			{
				// report exception to Output Console
				Print("Error in OnBarUpdate(): " + e.Message);
				// remember the error
				ErrorMessage = e.Message;
				// throw the error again
				throw;
			}
        }

		// updates given bar with data from www.tradingphysics.com 
		// if the data is not available, indicates if the session is 
		private BarUpdateResult TryUpdateBar(int ABarIndex)
		{
			// avoid error in NT 7.0 for BarsAgo greater than 255
			if (ABarIndex > 255) return BarUpdateResult.NOT_AVAILABLE;

			// session day for the current bar in the YYYYMMDD format
			string t = Time[ABarIndex].ToString("yyyyMMdd");
			
			// retrieve the day session either from the session manager's cache 
			// ...or from the www.tradingphysics.com web site in the background thread
			TradingPhysicsConnection.Session s;
			if (!MySessionManager.HasSession(t))
			{
				// request new session (the data retrieval will start in the background)
				s = MySessionManager.AddSession(t);
				try
				{
					s.Connect();
				}
				catch 
				{
					// error message, if any, willbe retrieved by calling the GetErrorMessage below
				}
			}
			else
			{
				// session is already requested, get it from the cache
				s = MySessionManager.GetSession(t);
			}

			// check if the background thread of this session terminated with an exception?
			if (s.GetErrorMessage() != "") 
			{
				// print error message to the debug window
				Print("Session " + t + " ended with an exception: " + s.GetErrorMessage());
				// don't display this error message again
				s.ClearErrorMessage();
			}
			
			// get index for the current bar time stamp, -1 if not available
			int n = s.ConvertDateTimeToTickIndex(Time[ABarIndex]);
			
			// is bar available in this session?
			if (n >= 0)
			{
				// yes -- get the value
				double v = s.GetIndicatorValue(MyIndicatorName, n);
				
				// set data and plot properties
				MyDataSeries.Set(ABarIndex, v);
				Plot0.Set(ABarIndex, v);
				
				// bar updated
				return BarUpdateResult.SUCCESS;
			}
			else 
			{
				// bar not available
				return (s.IsConnected()) ? BarUpdateResult.STILL_LOADING : BarUpdateResult.NOT_AVAILABLE;
			}
				
		}
		
		// Necessary to override in order to clean up resources used by the StreamReader object
		public override void Dispose() 
		{
			// call to SessionManager.Abort() is required to immediately terminate the background update threads, if any
			if (MySessionManager != null)
			{
				MySessionManager.Abort();
				MySessionManager = null;
			}
			
			// Required disposal call when overriding the Dispose() method
			base.Dispose();
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("Refer to www.tradingphysics.com for valid Indicator and Oscillator names")]
        [Category("Parameters")]
        public string AName
        {
            get { return aName; }
            set { aName = value; }
        }

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TradingPhysicsIndicator[] cacheTradingPhysicsIndicator = null;

        private static TradingPhysicsIndicator checkTradingPhysicsIndicator = new TradingPhysicsIndicator();

        /// <summary>
        /// Order-flow based indicators from www.tradingphysics.com
        /// </summary>
        /// <returns></returns>
        public TradingPhysicsIndicator TradingPhysicsIndicator(string aName)
        {
            return TradingPhysicsIndicator(Input, aName);
        }

        /// <summary>
        /// Order-flow based indicators from www.tradingphysics.com
        /// </summary>
        /// <returns></returns>
        public TradingPhysicsIndicator TradingPhysicsIndicator(Data.IDataSeries input, string aName)
        {
            if (cacheTradingPhysicsIndicator != null)
                for (int idx = 0; idx < cacheTradingPhysicsIndicator.Length; idx++)
                    if (cacheTradingPhysicsIndicator[idx].AName == aName && cacheTradingPhysicsIndicator[idx].EqualsInput(input))
                        return cacheTradingPhysicsIndicator[idx];

            lock (checkTradingPhysicsIndicator)
            {
                checkTradingPhysicsIndicator.AName = aName;
                aName = checkTradingPhysicsIndicator.AName;

                if (cacheTradingPhysicsIndicator != null)
                    for (int idx = 0; idx < cacheTradingPhysicsIndicator.Length; idx++)
                        if (cacheTradingPhysicsIndicator[idx].AName == aName && cacheTradingPhysicsIndicator[idx].EqualsInput(input))
                            return cacheTradingPhysicsIndicator[idx];

                TradingPhysicsIndicator indicator = new TradingPhysicsIndicator();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.AName = aName;
                Indicators.Add(indicator);
                indicator.SetUp();

                TradingPhysicsIndicator[] tmp = new TradingPhysicsIndicator[cacheTradingPhysicsIndicator == null ? 1 : cacheTradingPhysicsIndicator.Length + 1];
                if (cacheTradingPhysicsIndicator != null)
                    cacheTradingPhysicsIndicator.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTradingPhysicsIndicator = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Order-flow based indicators from www.tradingphysics.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TradingPhysicsIndicator TradingPhysicsIndicator(string aName)
        {
            return _indicator.TradingPhysicsIndicator(Input, aName);
        }

        /// <summary>
        /// Order-flow based indicators from www.tradingphysics.com
        /// </summary>
        /// <returns></returns>
        public Indicator.TradingPhysicsIndicator TradingPhysicsIndicator(Data.IDataSeries input, string aName)
        {
            return _indicator.TradingPhysicsIndicator(input, aName);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Order-flow based indicators from www.tradingphysics.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TradingPhysicsIndicator TradingPhysicsIndicator(string aName)
        {
            return _indicator.TradingPhysicsIndicator(Input, aName);
        }

        /// <summary>
        /// Order-flow based indicators from www.tradingphysics.com
        /// </summary>
        /// <returns></returns>
        public Indicator.TradingPhysicsIndicator TradingPhysicsIndicator(Data.IDataSeries input, string aName)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TradingPhysicsIndicator(input, aName);
        }
    }
}
#endregion
